#include "ObjectPlacer.h"

namespace GtpVisibilityPreprocessor {

// ---------------------------------------------------------------------------
//  ObjectPlacer implementation
// ---------------------------------------------------------------------------
#if 0
ObjectPlacer::ObjectPlacer()
	:mCurrentLibaryObject(0), mPlacedNodes(new BranchGroup), mLibraryViz(new BranchGroup)
{
	mLibraryViz->addChild(new BranchGroup);

	BranchGroup *bg = new BranchGroup;
	bg->addChild(new Box);
	SetLibrary(bg);
}


void ObjectPlacer::SetLibrary(BranchGroup *library)
{
	mLibrary = new BranchGroup;
	if (!library)
	{
		mLibrary->addChild(new Box);
	}
	else
	{
		for (int i = 0; i < library->numChildren(); i ++)
		{	
			Node *n = library->getChild(i);
			SharedGroup *s = new SharedGroup;
			s->addChild(n);
			
			Link *l = new Link;
			l->setSharedGroup(s);
			mLibrary->addChild(l);
		}
	}
	
	UpdateLibraryViz();
}

void ObjectPlacer::UpdateLibraryViz()
{
	Link *l = new Link;
	l->setSharedGroup(((Link *)mLibrary->getChild(mCurrentLibaryObject))->getSharedGroup());

	mLibraryViz->setChild(l, 0);
}


void ObjectPlacer::NextObject()
{
	if (mLibrary)
		mCurrentLibaryObject = (mCurrentLibaryObject + 1) % (mLibrary->numChildren());
	UpdateLibraryViz();
}

void ObjectPlacer::PreviousObject()
{
	if (mLibrary)
		mCurrentLibaryObject = (mCurrentLibaryObject - 1) % (mLibrary->numChildren());
	UpdateLibraryViz();
}

void ObjectPlacer::AddObject(const Point3f &p)
{
	//BranchGroup *o = new BranchGroup;
	TransformGroup *tg = new TransformGroup;
	Transform3D t;
	t.setTranslation(p - Origin3f);
	tg->setTransform(t);
	//o->addChild(tg);

	Link *l = new Link;
	l->setSharedGroup(((Link *)(mLibrary->getChild(mCurrentLibaryObject)))->getSharedGroup());
	tg->addChild(l);

	mPlacedNodes->addChild(tg);
}

void ObjectPlacer::RemoveLastObject()
{
	if (mPlacedNodes->numChildren() > 0)
		mPlacedNodes->removeChild(mPlacedNodes->numChildren()-1);
}

void ObjectPlacer::SaveObjects(const String &filename)
{
	WriteActionIV wa(filename, "");

	wa.Apply(mPlacedNodes);
}

BranchGroup *ObjectPlacer::GetLibraryVisualization()
{
	return mLibraryViz;
}

BranchGroup *ObjectPlacer::GetPlacedNodeVisualization()
{
	return mPlacedNodes;
}

void ObjectPlacer::Clear()
{
	mPlacedNodes->removeAllChildren();	
}


void TestApp::Placer(const String &parm)
{
	if (parm.Empty())
	{
		OUT1("Object placement: place/load <libaryfile>/removelast/next/previous/save <objectfile>");
		return;
	}

	ASM_RET(viewer->GetRoot(), "No scene");
	
	if (!placer)
		placer = new ObjectPlacer;

	//-- make sure the placer visualization (which shows the already placed objects) is in the viewer
	SearchNodeByNameVisitor search("placervis");
	search.Apply(viewer->GetRoot(), SearchNodeByNameVisitor::FIRST);
	BranchGroup *placervis = search.GetNode();
	if (!placervis)
	{
		placervis = placer->GetPlacedNodeVisualization();
		viewer->GetRoot()->addChild(placervis);
		placervis->setName("placervis");
	}

	if (parm == "place")
	{
		int x, y;
		input_module->GetMousePos(x, y);

		Point3f pos = viewer->UnProject(x, y);
		if (viewer && viewer->GetRoot() && viewer->GetRoot()->getBoundingBox()->intersects(pos))
		{
			placer->AddObject(pos);
			OUT1(x << " " << y << " " << pos);
		}
		else
		{
			OUT1("Point out of bounds");
		}
	} else if (parm == "removelast") {
		placer->RemoveLastObject();
	} else if (parm == "next") {
		placer->NextObject();
	} else if (parm == "previous") {
		placer->PreviousObject();
	} else if (parm.Left(5) == "save ") {
		String filename = parm.Right(parm.size()-5);
		if (!filename.Empty())
			placer->SaveObjects(filename);
	} else if (parm.Left(5) == "clear") {
		placer->Clear();
	} else if (parm.Left(5) == "load ") {
		String fn = sysutils::FindFile(parm.Right(parm.size()-5), settings->get_ObjectPath());

		ASM_ELSE(!fn.empty(), "Couldn't find file " << fn)
		{
			OUT1("Loading library " << fn << "...");
			SceneBasePtr<BranchGroup> root = ReadSceneGraph(fn, 0, 0, 1);

			BranchGroup *library = new BranchGroup;
			ASM_ELSE((root != NULL), "No scene found in file...")
			{
				SearchNodeByNameVisitor search("start");
				search.Apply(root, SearchNodeByNameVisitor::FIRST);
				BranchGroup *local_root = search.GetNode();

				if (local_root == NULL)
				{
					OUT1("Note: BranchGroup named \"start\" not found - using root node");
					local_root = root;
				}

				placer->SetLibrary(local_root);
				BranchGroup *viz = placer->GetLibraryVisualization();
				mRenderArea->SetSceneRoot(viz);
				mRenderArea->SetCamera(new Camera);
				mRenderArea->GetCamera()->ViewAll(viz->getBoundingSphere());
			}
		}
	} else
		OUT1("Unknown placer command");

}

#endif

}